from debiai_data_provider.data_provider import DataProvider
from debiai_data_provider.version import VERSION


def start_api_server(data_provider: DataProvider, host, port):
    import uvicorn
    from fastapi import FastAPI
    from fastapi.middleware.cors import CORSMiddleware
    from debiai_data_provider.controller.routes import router as controller_router

    app = FastAPI(
        title="DebiAI Data-provider API",
        version=VERSION,
        description="API for DebiAI data providers",
    )

    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    app.state.data_provider = data_provider

    app.include_router(controller_router)

    uvicorn.run(app, host=host, port=port)



from debiai_data_provider.models.project import DebiAIProject
from debiai_data_provider.models.debiai import Column
import pandas as pd
from typing import List


def extract_project_class_name(project: DebiAIProject) -> str:
    # Get the class name
    return project.__class__.__name__


def dataframe_to_debiai_data_array(
    columns: List[Column],
    samples_id: List[str],
    data: pd.DataFrame,
):
    sample_dicts = {}
    for sample_id in samples_id:
        sample_data = []

        for column in columns:
            try:
                if "Data ID" in data.columns:
                    sample_to_add = data.loc[
                        data["Data ID"] == sample_id, column.name
                    ].values[0]
                else:
                    # Use the dataframe index as the sample ID
                    sample_to_add = data.loc[sample_id, column.name]
            except KeyError:
                raise KeyError(
                    f"Column '{column.name}' of the sample '{sample_id}' not found in the data."
                )

            sample_data.append(sample_to_add)

        sample_dicts[sample_id] = sample_data

    return sample_dicts



from fastapi import APIRouter, Depends, Request
from typing import List, Dict, Optional, Union
from fastapi import Path, Query, Body
from debiai_data_provider.models.debiai import (
    InfoResponse,
    CanDelete,
    ProjectOverview,
    ProjectDetails,
    ModelDetail,
    SelectionRequest,
)
from debiai_data_provider.version import VERSION
from debiai_data_provider.data_provider import DataProvider

router = APIRouter()


def get_data_provider(request: Request):
    return request.app.state.data_provider


# Info routes
@router.get("/info", response_model=InfoResponse, tags=["Info"])
def get_info(data_provider: DataProvider = Depends(get_data_provider)):
    return InfoResponse(
        version=VERSION,
        maxSampleIdByRequest=data_provider.max_sample_id_by_request or 10000,
        maxSampleDataByRequest=data_provider.max_sample_data_by_request or 2000,
        maxResultByRequest=data_provider.max_result_by_request or 5000,
        canDelete=CanDelete(),
    )


# Project routes
@router.get("/projects", response_model=Dict[str, ProjectOverview], tags=["Projects"])
def get_projects(data_provider: DataProvider = Depends(get_data_provider)):
    debiai_projects = data_provider.projects
    return {project.project_name: project.get_overview() for project in debiai_projects}


@router.get("/projects/{projectId}", response_model=ProjectDetails, tags=["Projects"])
def get_project(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    data_provider: DataProvider = Depends(get_data_provider),
):
    return data_provider._get_project_to_expose(projectId).get_details()


@router.delete("/projects/{projectId}", status_code=200, tags=["Projects"])
def delete_project(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    data_provider: DataProvider = Depends(get_data_provider),
):
    data_provider.delete_project(projectId)
    return {"message": "Project deleted"}


# Data routes
@router.get(
    "/projects/{projectId}/data-id-list",
    response_model=List[Union[str, int]],
    tags=["Data"],
)
def get_data_id_list(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    from_: Optional[int] = Query(None, alias="from"),
    to: Optional[int] = Query(None),
    analysisId: Optional[str] = Query(None),
    analysisStart: Optional[bool] = Query(None),
    analysisEnd: Optional[bool] = Query(None),
    data_provider: DataProvider = Depends(get_data_provider),
):
    project = data_provider._get_project_to_expose(projectId)
    return project.get_data_id_list(from_, to, analysisId, analysisStart, analysisEnd)


@router.post(
    "/projects/{projectId}/data",
    response_model=Dict[
        Union[str, int], List[Union[str, int, float, bool, None, list, dict]]
    ],
    tags=["Data"],
)
def get_data(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    samples_ids: List[Union[str, int, float]] = Body(...),
    data_provider: DataProvider = Depends(get_data_provider),
):
    project = data_provider._get_project_to_expose(projectId)
    return project.get_data_from_ids(samples_ids)


# Model routes
@router.get(
    "/projects/{projectId}/models",
    response_model=List[ModelDetail],
    tags=["Models"],
)
def get_models(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    data_provider: DataProvider = Depends(get_data_provider),
):
    project = data_provider._get_project_to_expose(projectId)
    return project.get_models()


@router.get(
    "/projects/{projectId}/models/{modelId}/evaluated-data-id-list",
    response_model=List[Union[str, int]],
    tags=["Models"],
)
def get_models_evaluated_data_id_list(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    modelId: str = Path(..., min_length=1, example="Model 1"),
    data_provider: DataProvider = Depends(get_data_provider),
):
    project = data_provider._get_project_to_expose(projectId)
    return project.get_model_evaluated_data_id_list(modelId)


@router.post(
    "/projects/{projectId}/models/{modelId}/results",
    response_model=Dict[Union[str, int], List[Union[str, int, float, bool]]],
    tags=["Models"],
)
def get_model_results(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    modelId: str = Path(..., min_length=1, example="Model 1"),
    body: List[Union[str, int, float]] = Body(...),
    data_provider: DataProvider = Depends(get_data_provider),
):
    project = data_provider._get_project_to_expose(projectId)
    return project.get_model_results(modelId, body)


@router.delete(
    "/projects/{projectId}/models/{modelId}", status_code=204, tags=["Models"]
)
def delete_model(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    modelId: str = Path(..., min_length=1, example="Model 1"),
):
    return {"message": "Model deleted"}


# Selection routes
@router.get(
    "/projects/{projectId}/selections",
    response_model=List[Dict[str, Union[str, int]]],
    tags=["Selections"],
)
def get_selections(projectId: str = Path(..., min_length=1, example="Project 1")):
    return []


@router.post("/projects/{projectId}/selections", status_code=204, tags=["Selections"])
def create_selection(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    body: SelectionRequest = Body(...),
):
    return {"message": "Selection created successfully"}


@router.delete(
    "/projects/{projectId}/selections/{selectionId}",
    status_code=204,
    tags=["Selections"],
)
def delete_selection(
    projectId: str = Path(..., min_length=1, example="Project 1"),
    selectionId: str = Path(..., min_length=1, example="Selection 1"),
):
    return {"message": "Selection deleted"}



import os
import pandas as pd
import numpy as np
from debiai_data_provider.models.project import DebiAIProject
from pydantic import BaseModel, Field
from typing import List, Optional
from pathlib import Path
from rich.console import Console
from rich.table import Table

# A Data-provider that only needs a path to a parquet file


class ParquetDataProviderConfig(BaseModel):
    parquet_path: str = Field(..., description="Path to the parquet file")
    sample_id_column_name: str = Field(
        ...,
        description="Name of the column containing the sample IDs, for the samples and results",
    )
    name: Optional[str] = Field(None, description="Name of the project")
    columns: Optional[List[str]] = Field(
        None, description="Columns to include in the project"
    )
    results_parquet_folder_path: Optional[str] = Field(  # Renamed field
        None, description="Path to the folder containing model results"
    )
    results_columns: Optional[List[str]] = Field(
        None, description="Columns for model results"
    )
    ignored_columns: Optional[List[str]] = Field(
        None, description="Columns to ignore in the project"
    )
    ignored_results_columns: Optional[List[str]] = Field(
        None, description="Columns to ignore in model results"
    )


class ParquetDataProvider(DebiAIProject):
    creation_date = "2025-03-28"
    # update_date = "2025-03-28"
    data: pd.DataFrame = None
    model_results: pd.DataFrame = None

    def __init__(
        self,
        parquet_path: str,
        sample_id_column_name: str,
        name: Optional[str] = None,
        columns: Optional[List[str]] = None,
        results_parquet_folder_path: Optional[str] = None,
        results_columns: Optional[List[str]] = None,
        ignored_columns: Optional[List[str]] = None,
        ignored_results_columns: Optional[List[str]] = None,
    ):
        super().__init__()
        self.config = ParquetDataProviderConfig(
            parquet_path=parquet_path,
            name=name,
            columns=columns,
            sample_id_column_name=sample_id_column_name,
            results_parquet_folder_path=results_parquet_folder_path,
            results_columns=results_columns,
            ignored_columns=ignored_columns,
            ignored_results_columns=ignored_results_columns,
        )

        # Setup name
        if self.config.name:
            self.name = self.config.name
        else:
            # Extract name from parquet_path
            self.name = Path(self.config.parquet_path).stem

        # Display project information
        console = Console()
        table = Table(title="Loading Parquet File")
        table.add_column("Property", style="cyan", no_wrap=True)
        table.add_column("Value", style="magenta")
        table.add_row("File Path", self.config.parquet_path)
        table.add_row("Project Name", self.name or "N/A")
        table.add_row(
            "Columns to Include",
            (
                "\n".join(f"  - {col}" for col in self.config.columns)
                if self.config.columns
                else "All"
            ),
        )
        table.add_row("Sample ID Column", self.config.sample_id_column_name)
        console.print(table)

        # Load the data from the parquet files
        self.load_project_parquet_samples()
        self.load_model_parquet_results()

    def load_project_parquet_samples(self):
        parquet_df = pd.read_parquet(self.config.parquet_path)

        # Check if the sample_id_column_name is in the columns
        if self.config.sample_id_column_name not in parquet_df.columns:
            console = Console()
            available_columns = "\n".join(f"  - {col}" for col in parquet_df.columns)
            console.print(
                f"[bold red]Error:[/bold red] Column '[cyan]{self.config.sample_id_column_name}[/cyan]'\
 not found in the parquet file.",
                style="red",
            )
            console.print(
                "[bold red]This column is required to identify the samples.[/bold red]",
                style="red",
            )
            console.print(
                f"[bold magenta]Available columns are:[/bold magenta]\n{available_columns}",
                style="magenta",
            )
            raise ValueError(
                f"Column '{self.config.sample_id_column_name}' not found in the parquet file."
            )

        # Check if all the sample IDs column values are type str
        for id in parquet_df[self.config.sample_id_column_name]:
            if not isinstance(id, str):
                console = Console()
                console.print(
                    "[bold red]Error:[/bold red] The sample IDs in the parquet file must be strings.",
                    style="red",
                )
                raise ValueError("Sample IDs must be strings.")

        # Check if the sample IDs are unique
        if not parquet_df[self.config.sample_id_column_name].is_unique:
            console = Console()
            console.print(
                "[bold red]Error:[/bold red] The sample IDs in the parquet file must be unique.",
                style="red",
            )
            raise ValueError("Sample IDs must be unique.")

        # Filter columns if specified
        if self.config.columns:
            columns_to_keep = set(self.config.columns)

            # Keep the sample_id_column_name if specified
            if self.config.sample_id_column_name:
                columns_to_keep.add(self.config.sample_id_column_name)

            parquet_df = parquet_df[list(columns_to_keep)]

        # Filter out ignored columns
        if self.config.ignored_columns:
            parquet_df = parquet_df.drop(
                columns=self.config.ignored_columns, errors="ignore"
            )

        # Validate data types and log faulty columns
        valid_types = (str, int, float, bool, list, dict)
        for column in parquet_df.columns:
            invalid_rows = parquet_df[column].apply(
                lambda x: not isinstance(x, valid_types)
            )
            if invalid_rows.any():
                # get whats the type of the first invalid value
                invalid_value_type = type(parquet_df[invalid_rows][column].iloc[0])

                console = Console()
                table = Table(
                    title=f"Invalid Data in Column '{column}'\nThis column might \
cause issues in DebiAI, you can use the `ignored_columns` parameter to ignore this column.",
                    show_header=True,
                    header_style="bold red",
                )
                table.add_column("Row Index", no_wrap=True)
                table.add_column("Value", style="red")
                table.add_row("Invalid Value Type", str(invalid_value_type))
                table.add_row("Invalid Rows Count", str(invalid_rows.sum()))

                for idx, value in parquet_df[invalid_rows][column].head(5).items():
                    table.add_row(str(idx), str(value))

                console.print(table)

        # Convert np.int64 to native Python int
        parquet_df = parquet_df.map(lambda x: int(x) if isinstance(x, np.int64) else x)

        # Store the data
        self.data = parquet_df

    def load_model_parquet_results(self):
        if not self.config.results_parquet_folder_path:
            return

        model_results = None

        for results_file in os.listdir(self.config.results_parquet_folder_path):
            if not results_file.endswith(".parquet"):
                continue

            model_name = results_file.split(".")[0]
            parquet_df = pd.read_parquet(
                os.path.join(self.config.results_parquet_folder_path, results_file)
            )

            # Check if the sample_id_column_name is in the columns
            if self.config.sample_id_column_name not in parquet_df.columns:
                console = Console()
                available_columns = "\n".join(
                    f"  - {col}" for col in parquet_df.columns
                )
                console.print(
                    f"[bold red]Error:[/bold red] Column \
'[cyan]{self.config.sample_id_column_name}[/cyan]' not found in the {model_name} parquet file.",
                    style="red",
                )
                console.print(
                    "[bold red]This column is required to map the model \
results to the samples.[/bold red]",
                    style="red",
                )
                console.print(
                    f"[bold magenta]Available columns are:[/bold magenta]\n{available_columns}",
                    style="magenta",
                )
                raise ValueError(
                    f"Column '{self.config.sample_id_column_name}' not found in the parquet file."
                )

            # Filter columns if specified
            if self.config.results_columns:
                columns_to_keep = set(self.config.results_columns)

                # Keep the sample_id_column_name if specified
                if self.config.sample_id_column_name:
                    columns_to_keep.add(self.config.sample_id_column_name)

                parquet_df = parquet_df[list(columns_to_keep)]

            # Filter out ignored columns
            if self.config.ignored_results_columns:
                parquet_df = parquet_df.drop(
                    columns=self.config.ignored_results_columns, errors="ignore"
                )

            # Add a _model_name column
            parquet_df["_model_name"] = model_name

            if model_results is None:
                model_results = parquet_df
            else:
                # Stack the results
                model_results = pd.concat(
                    [model_results, parquet_df], ignore_index=True
                )

        # Store the model results
        self.model_results = model_results

    # Project Info
    def get_structure(self) -> dict:
        # Load the data from the parquet file
        UNWANTED_COLUMNS = [self.config.sample_id_column_name]

        # Create the structure
        project_structure = {}

        for col in self.data.columns:
            if col in UNWANTED_COLUMNS:
                continue

            project_structure[col] = {
                "category": "context",
                "type": "auto",
            }

        return project_structure

    def get_results_structure(self) -> dict:
        # Load the data from the parquet file
        if not self.config.results_parquet_folder_path:
            raise NotImplementedError(
                "Results structure is not available for this project."
            )

        UNWANTED_COLUMNS = [self.config.sample_id_column_name]

        # Create the structure
        results_structure = {}

        # Iterate over the columns of the model_results DataFrame
        for col in self.model_results.columns:
            if col in UNWANTED_COLUMNS or col == "_model_name":
                continue

            results_structure[col] = {"type": "auto"}

        return results_structure

    # Project Samples
    def get_nb_samples(self) -> int:
        # This function returns the number of samples in the project
        return len(self.data)

    def get_samples_ids(self) -> List[str]:
        # This function returns the list of samples ids
        project_data = self.data
        return project_data[self.config.sample_id_column_name].tolist()

    def get_data(self, samples_ids: List[str]) -> pd.DataFrame:
        # This function will be called when the user
        # wants to analyze data from your project

        # The function should return a pandas DataFrame
        # containing the data corresponding to the samples_ids
        project_data = self.data.set_index(self.config.sample_id_column_name)
        data = project_data.loc[samples_ids]
        return data

    # Project models
    def get_models(self) -> List[dict]:
        # List the models available in the project
        if self.model_results is None:
            return []

        models = []
        # Get the list of models from model_results dataframe
        unique_model_names = self.model_results["_model_name"].unique()

        for model_name in unique_model_names:
            # Count the number of results for the model
            num_results = len(self.get_model_evaluated_data_id_list(model_name))

            models.append(
                {
                    "id": model_name,
                    "name": model_name,
                    "nb_results": num_results,
                }
            )

        # Sort the models by name
        models.sort(key=lambda x: x["name"])

        return models

    def get_model_evaluated_data_id_list(self, model_id: str) -> List[str]:
        # This function returns the list of sample IDs for a given model
        if not self.config.results_parquet_folder_path:
            return []

        # Filter the model results
        model_results = self.model_results[
            self.model_results["_model_name"] == model_id
        ]

        # Only keep the samples_id that are in the project data
        project_samples_id = self.get_samples_ids()
        model_results = model_results[
            model_results[self.config.sample_id_column_name].isin(project_samples_id)
        ]

        # Return the list of sample IDs
        return model_results[self.config.sample_id_column_name].tolist()

    def get_model_results(
        self, model_id: str, samples_ids: List[str]
    ) -> pd.DataFrame:  # noqa
        # Construct the path to the model's parquet file
        if not self.config.results_parquet_folder_path:
            return []

        # Filter the model results
        model_results = self.model_results[
            self.model_results["_model_name"] == model_id
        ]

        # Filter the results for the given sample IDs
        model_results = model_results[model_results["sample_id"].isin(samples_ids)]

        # Return the filtered DataFrame
        return model_results



from pydantic import BaseModel
from typing import List, Optional


class CanDelete(BaseModel):
    projects: bool = True
    selections: bool = True
    models: bool = True


class InfoResponse(BaseModel):
    version: str
    maxSampleIdByRequest: int = 10000
    maxSampleDataByRequest: int = 2000
    maxResultByRequest: int = 5000
    canDelete: CanDelete


class ProjectOverview(BaseModel):
    name: Optional[str]
    nbSamples: Optional[int] = None
    nbModels: Optional[int] = None
    nbSelections: Optional[int] = None
    creationDate: Optional[int] = None
    updateDate: Optional[int] = None


class Column(BaseModel):
    name: str
    category: str = "other"
    type: Optional[str] = "auto"
    group: Optional[str] = None


class ExpectedResult(BaseModel):
    name: str
    type: str = "auto"
    group: Optional[str] = ""


class ProjectDetails(BaseModel):
    name: Optional[str]
    columns: List[Column]
    expectedResults: List[ExpectedResult]
    nbSamples: Optional[int] = None
    creationDate: Optional[int] = None
    updateDate: Optional[int] = None


class ModelDetail(BaseModel):
    id: str
    name: Optional[str]
    nbResults: Optional[int] = None
    creationDate: Optional[int] = None


class SelectionRequest(BaseModel):
    name: str
    idList: List[str]



import pandas as pd
from rich.table import Table
from debiai_data_provider.models.debiai import (
    ProjectOverview,
    ProjectDetails,
    ModelDetail,
    Column,
    ExpectedResult,
)
from typing import Optional, Union, List, Tuple, Dict


class DebiAIProject:
    creation_date: Optional[Union[None, str]] = None
    update_date: Optional[Union[None, str]] = None
    name: Optional[str] = None

    # Project information
    def get_structure(self) -> dict:
        raise NotImplementedError

    def get_results_structure(self) -> dict:
        return {}

    # Project actions
    def delete_project(self):
        raise NotImplementedError

    # Project Samples
    def get_nb_samples(self) -> Union[int, None]:
        return None

    def get_samples_ids(self) -> List[str]:
        raise NotImplementedError

    def get_data(self, samples_ids: List[str]) -> pd.DataFrame:
        raise NotImplementedError

    # Project models
    def get_models(self) -> List[ModelDetail]:
        return []

    def get_model_evaluated_data_id_list(self, model_id: str) -> List[str]:
        raise NotImplementedError

    def get_model_results(self, model_id: str, sample_ids: List[str]) -> pd.DataFrame:
        raise NotImplementedError


class ProjectToExpose:
    def __init__(self, project: DebiAIProject, project_name: str):
        self.project = project
        self.project_name = project_name

    # Getters
    def get_columns(self) -> Union[List[Column], None]:
        try:
            structure = self.project.get_structure()
        except NotImplementedError:
            return None

        if not isinstance(structure, dict):
            raise ValueError("The 'get_structure' method must return a dictionary.")

        structure = structure.copy()

        for key, value in structure.items():
            if not isinstance(value, dict):
                raise ValueError(
                    f"Error in the structure of the column '{key}', it must be a dictionary."
                )

            if "category" not in value:
                # Set the default category to "other"
                value["category"] = "other"
            else:
                if not isinstance(value["category"], str):
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'category' must be a string."
                    )

                if value["category"] not in [
                    "context",
                    "input",
                    "groundtruth",
                    "other",
                ]:
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'category' must be 'context', 'input', 'groundtruth' or 'other'."  # noqa
                    )

            if "type" in value:
                if not isinstance(value["type"], str):
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'type' must be a string."
                    )

                VALID_TYPES = ["text", "number", "bool", "dict", "list", "auto"]
                if value["type"] not in VALID_TYPES:
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'type' must be "
                        + ", ".join(VALID_TYPES)
                        + "."
                    )

            if "group" in value:
                if not isinstance(value["group"], str):
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'group' must be a string."
                    )

        # Convert:
        # {
        #     "col_name": {
        #         "type": "text",
        #         "category": "context",
        #         "group": "context",
        #     },
        #     ...
        # }
        # to:
        # [
        #     Column(name="col_name", type="text", category="context", group="context"),
        # ]

        columns = []
        for key, value in structure.items():
            columns.append(
                Column(
                    name=key,
                    category=value["category"],
                    type=value["type"],
                    group=value.get("group", ""),
                )
            )

        return columns

    def get_results_columns(self) -> Union[List[ExpectedResult], None]:
        try:
            structure = self.project.get_results_structure()
        except NotImplementedError:
            return None

        if not isinstance(structure, dict):
            raise ValueError(
                f'The "get_results_structure" method must return a dictionary, got {type(structure)}.\n\
Expected dictionary format: {{"col_name": {{"type": text, "group": text}}}}.'
            )

        structure = structure.copy()

        for key, value in structure.items():
            if not isinstance(value, dict):
                raise ValueError(
                    f"Error in the structure of the column '{key}', it must be a dictionary."
                )

            if "type" in value:
                if not isinstance(value["type"], str):
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'type' must be a string."
                    )

                VALID_TYPES = ["text", "number", "bool", "dict", "list", "auto"]
                if value["type"] not in VALID_TYPES:
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'type' must be "
                        + ", ".join(VALID_TYPES)
                        + "."
                    )

            if "group" in value:
                if not isinstance(value["group"], str):
                    raise ValueError(
                        f"Error in the structure of the column '{key}', the 'group' must be a string."
                    )
        # Convert:
        # {
        #     "col_name": {
        #         "type": "text",
        #         "group": "context",
        #     },
        #     ...
        # }
        # to:
        # [
        #     ExpectedResult(name="col_name", type="text", group="context"),
        # ]

        columns = []
        for key, value in structure.items():
            columns.append(
                ExpectedResult(
                    name=key,
                    type=value["type"],
                    group=value.get("group", ""),
                )
            )

        return columns

    def get_nb_samples(self) -> Union[int, None]:
        nb_samples = self.project.get_nb_samples()

        if nb_samples is None or not isinstance(nb_samples, int):
            return None

        return nb_samples

    def get_samples_ids(self) -> List[str]:
        try:
            samples_id = self.project.get_samples_ids()
        except NotImplementedError:
            return []

        if not isinstance(samples_id, list):
            raise ValueError("The 'get_samples_ids' method must return a list.")

        # Ids must be strings or integers
        if not all(isinstance(x, (str, int)) for x in samples_id):
            raise ValueError(
                "The 'get_samples_ids' method must return a list of strings."
            )

        return samples_id

    # Project information
    def get_dates(self) -> Tuple[Optional[int], Optional[int]]:
        # Get the creation date
        creationDate = None
        if self.project.creation_date is not None and isinstance(
            self.project.creation_date, str
        ):
            # Convert the creation date to a timestamp
            creationDate = pd.Timestamp(self.project.creation_date).timestamp() * 1000

        # Get update date
        updateDate = None
        if self.project.update_date is not None and isinstance(
            self.project.update_date, str
        ):
            # Convert the update date to a timestamp
            updateDate = pd.Timestamp(self.project.update_date).timestamp() * 1000

        return creationDate, updateDate

    def get_overview(self) -> ProjectOverview:
        # Get project details
        creationDate, updateDate = self.get_dates()

        # Get the number of samples
        nbSamples = self.get_nb_samples()

        # Get models details
        models = self.get_models()

        return ProjectOverview(
            name=self.project_name,
            nbSamples=nbSamples,
            nbModels=len(models),
            nbSelections=None,
            creationDate=creationDate,
            updateDate=updateDate,
        )

    def get_details(self) -> ProjectDetails:
        # Get project details
        creationDate, updateDate = self.get_dates()

        # Get the number of samples
        nbSamples = self.get_nb_samples()

        # Construct the project columns
        columns = self.get_columns()
        results_columns = self.get_results_columns()

        return ProjectDetails(
            name=self.project_name,
            columns=columns,
            expectedResults=results_columns,
            nbSamples=nbSamples,
            creationDate=creationDate,
            updateDate=updateDate,
        )

    # Samples
    def get_data_id_list(
        self,
        from_: Optional[int] = None,
        to: Optional[int] = None,
        analysisId: Optional[str] = None,
        analysisStart: Optional[bool] = None,
        analysisEnd: Optional[bool] = None,
    ) -> List[str]:
        samples_ids = self.get_samples_ids()

        if from_ is not None and to is not None:
            samples_ids = samples_ids[from_ : to + 1]  # noqa

        elif from_ is not None:
            samples_ids = samples_ids[from_:]

        elif to is not None:
            samples_ids = samples_ids[: to + 1]

        return samples_ids

    def get_data_from_ids(self, samples_ids: List[str]) -> dict:
        from debiai_data_provider.utils.parser import dataframe_to_debiai_data

        # Get the data from the project
        df_data = self.project.get_data(samples_ids)

        # Create a copy of the dataframe
        df_data = df_data.copy()

        # Verify that all the columns are in the dataframe
        columns = self.get_columns()
        if not columns:
            raise ValueError("The project has no columns defined.")

        for column in columns:
            if column.name not in df_data.columns:
                # Add the column to the dataframe
                df_data[column.name] = None

        return dataframe_to_debiai_data(
            columns=columns, samples_id=samples_ids, data=df_data
        )

    # Models
    def get_models(self) -> List[ModelDetail]:
        models = self.project.get_models()

        # Convert the models to ModelDetail
        model_details = []
        for model in models:
            if isinstance(model, ModelDetail):
                model_details.append(model)
            elif isinstance(model, dict):
                if "id" not in model:
                    raise ValueError("The model must have an 'id' key.")

                model_details.append(
                    ModelDetail(
                        id=model["id"],
                        name=model.get("name", None),
                        nbResults=model.get("nb_results", None),
                        creationDate=model.get("creation_date", None),
                    )
                )
            else:
                raise ValueError("The model must be a ModelDetail or a dictionary.")

        return model_details

    def get_model_evaluated_data_id_list(self, model_id: str) -> List[str]:
        return self.project.get_model_evaluated_data_id_list(model_id)

    def get_model_results(
        self, model_id: str, sample_ids: List[str]
    ) -> Dict[str, list]:
        df_results = self.project.get_model_results(model_id, sample_ids)

        # Convert the dataframe to a list of dictionaries
        results = df_results.to_dict(orient="records")
        # Results:
        # {
        #     s_id: {
        #         "res_col_1": "OK",
        #         "res_col_2": 0.05,
        #         "res_col_3": 0.94,
        #         ...
        #     },
        #     "..."
        # }

        # Convert to list with same order as results columns:
        # {
        #     s_id: ["OK", 0.05, 0.94, ...],
        #     "..."
        # }
        results_dict = {}
        results_columns = self.get_results_columns()

        for sample_id, result in zip(sample_ids, results):
            results_list = []

            for column in results_columns:
                if column.name in result:
                    results_list.append(result[column.name])
                else:
                    results_list.append(None)

            results_dict[sample_id] = results_list

        return results_dict

    # Other
    def get_rich_table(self):
        # Display the Project details
        table = Table(width=80)
        table.add_column(
            self.project_name, style="cyan", no_wrap=True, justify="right", width=20
        )

        # Get updated and creation date
        creation_update_text = ""
        if self.project.creation_date:
            creation_update_text += (
                f"Created: {pd.Timestamp(self.project.creation_date).date()} "
            )
        if self.project.update_date:
            creation_update_text += (
                f"Updated: {pd.Timestamp(self.project.update_date).date()}"
            )
        table.add_column(creation_update_text, width=60)

        # Display the project column structure
        columns = self.get_columns()
        if columns:
            table.add_row("Structure:", "")
            for column in columns:
                column_value = (
                    f"[bold blue]{column.type}[/bold blue] "
                    + f"[italic]{column.category}[/italic]"
                )
                if column.group:
                    column_value += f" [blue]<{column.group}>[/blue]"

                table.add_row(
                    f"[bold green]{column.name}[/bold green]",
                    column_value,
                )
            table.add_row("", "")

        # Display the project data number of samples
        nb_samples = self.get_nb_samples()
        if nb_samples is not None:
            table.add_row("NB samples:", f"{nb_samples}")
            table.add_row("", "")

        # Display the project results structure
        results_columns = self.get_results_columns()
        if results_columns:
            table.add_row("Results:", "")
            for column in results_columns:
                column_value = f"[bold blue]{column.type}[/bold blue] "
                if column.group:
                    column_value += f" [blue]<{column.group}>[/blue]"

                table.add_row(
                    f"[bold green]{column.name}[/bold green]",
                    column_value,
                )
            table.add_row("", "")

        # Display the project models
        models = self.get_models()
        if models:
            table.add_row("Models:", "")
            for model in models:
                model_value = f"[bold blue]{model.id}[/bold blue]"
                if model.name:
                    model_value += f" [italic]{model.name}[/italic]"
                if model.creationDate:
                    model_value += f" ({pd.Timestamp(model.creationDate).date()})"
                if model.nbResults:
                    model_value += f" ({model.nbResults} results)"

                table.add_row(
                    f"[bold green]{model.id}[/bold green]",
                    model_value,
                )
            table.add_row("", "")

        return table



import pandas as pd
from debiai_data_provider.providers.parquet_data_provider import ParquetDataProvider
from tests.test_utils import create_temp_parquet_file, create_temp_results_folder
import pytest


def test_bad_configurations():
    # Non-existing parquet_path
    with pytest.raises(FileNotFoundError, match="No such file or directory"):
        ParquetDataProvider(
            parquet_path="non-existing", sample_id_column_name="sample_id"
        )

    data = pd.DataFrame(
        {
            "sample_id": ["1", "2", "3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )

    with create_temp_parquet_file(data) as parquet_path:
        # Non existing sample_id_column_name
        with pytest.raises(ValueError, match="Column 'non-existing' not found"):
            ParquetDataProvider(
                parquet_path=parquet_path, sample_id_column_name="non-existing"
            )

        # Results folder path does not exist
        with pytest.raises(
            FileNotFoundError, match="No such file or directory: 'non-existing'"
        ):
            ParquetDataProvider(
                parquet_path=parquet_path,
                sample_id_column_name="sample_id",
                results_parquet_folder_path="non-existing",
            )
        # Results folder path is not a directory
        with pytest.raises(NotADirectoryError, match="Not a directory"):
            ParquetDataProvider(
                parquet_path=parquet_path,
                sample_id_column_name="sample_id",
                results_parquet_folder_path=parquet_path,
            )

    # Samples IDs must be strings
    data = pd.DataFrame(
        {
            "sample_id": [1, 2, 3],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    with create_temp_parquet_file(data) as parquet_path:
        with pytest.raises(ValueError, match="Sample IDs must be strings"):
            ParquetDataProvider(
                parquet_path=parquet_path, sample_id_column_name="sample_id"
            )

    # Sample IDs must be unique
    data = pd.DataFrame(
        {
            "sample_id": ["1", "2", "1"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    with create_temp_parquet_file(data) as parquet_path:
        with pytest.raises(ValueError, match="Sample IDs must be unique"):
            ParquetDataProvider(
                parquet_path=parquet_path, sample_id_column_name="sample_id"
            )

    # results with non-existing sample_id
    data = pd.DataFrame(
        {
            "sample_id": ["S1", "S2", "S3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    results = {
        "m1": pd.DataFrame(
            {
                "predicted_state": ["OK", "KO"],
                "score": [0.9, 0.8],
            }
        ),
        "m2": pd.DataFrame(
            {
                "predicted_state": ["KO", "OK"],
                "score": [0.7, 0.6],
            }
        ),
    }
    with create_temp_parquet_file(data) as parquet_path, create_temp_results_folder(
        results
    ) as results_dir:
        with pytest.raises(
            ValueError,
            match="Column 'sample_id' not found",
        ):
            ParquetDataProvider(
                parquet_path=parquet_path,
                sample_id_column_name="sample_id",
                results_parquet_folder_path=results_dir,
            )


def test_parquet_data_provider_basic():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["s1", "s2", "s3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    with create_temp_parquet_file(data) as parquet_path:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
        )

        # Test get_nb_samples
        assert provider.get_nb_samples() == 3

        # Test get_samples_ids
        assert provider.get_samples_ids() == ["s1", "s2", "s3"]

        # Test get_data
        returned_data = provider.get_data(["s1", "s2"])
        assert returned_data.index.tolist() == ["s1", "s2"]
        assert returned_data["class"].tolist() == ["A", "B"]
        assert returned_data["value"].tolist() == [10, 20]

        # Test ges models and results (should be empty)
        assert provider.get_models() == []
        assert provider.get_model_results("m1", ["s1"]) == []
        assert provider.get_model_evaluated_data_id_list("m1") == []

        # get_results_structure should rise not implemented error
        with pytest.raises(NotImplementedError):
            provider.get_results_structure()

        # Test custom name
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            name="test_name",
        )
        assert provider.name == "test_name"


def test_parquet_data_provider_with_results():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["S1", "S2", "S3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    results = {
        "m1": pd.DataFrame(
            {
                "sample_id": ["S1", "S2"],
                "predicted_state": ["OK", "KO"],
                "score": [0.9, 0.8],
            }
        ),
        "m2": pd.DataFrame(
            {
                "sample_id": ["S2", "S3"],
                "predicted_state": ["KO", "OK"],
                "score": [0.7, 0.6],
            }
        ),
    }
    with create_temp_parquet_file(data) as parquet_path, create_temp_results_folder(
        results
    ) as results_dir:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            results_parquet_folder_path=results_dir,
        )

        # Test get_models
        models = provider.get_models()
        assert len(models) == 2
        assert models[0]["id"] == "m1"
        assert models[0]["name"] == "m1"
        assert models[1]["id"] == "m2"
        assert models[1]["name"] == "m2"

        # Test get_model_evaluated_data_id_list
        assert provider.get_model_evaluated_data_id_list("m1") == ["S1", "S2"]
        assert provider.get_model_evaluated_data_id_list("m2") == ["S2", "S3"]

        # Test get_model_results
        m1_results = provider.get_model_results("m1", ["S1", "S2"])
        assert m1_results["sample_id"].tolist() == ["S1", "S2"]
        assert m1_results["predicted_state"].tolist() == ["OK", "KO"]
        assert m1_results["score"].tolist() == [0.9, 0.8]

        m2_results = provider.get_model_results("m2", ["S2", "S3"])
        assert m2_results["sample_id"].tolist() == ["S2", "S3"]
        assert m2_results["predicted_state"].tolist() == ["KO", "OK"]
        assert m2_results["score"].tolist() == [0.7, 0.6]

        # New provider with columns filter
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            results_parquet_folder_path=results_dir,
            results_columns=["predicted_state"],
        )
        m1_results = provider.get_model_results("m1", ["S1", "S2"])
        assert m1_results["sample_id"].tolist() == ["S1", "S2"]
        assert "predicted_state" in m1_results.columns
        assert "score" not in m1_results.columns


def test_parquet_data_provider_with_columns():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["1", "2", "3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    with create_temp_parquet_file(data) as parquet_path:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            columns=["class"],
        )

        # Test get_data with filtered columns
        data = provider.get_data(["1", "2"])
        assert data.index.tolist() == ["1", "2"]
        assert "class" in data.columns
        assert "value" not in data.columns
        assert data["class"].tolist() == ["A", "B"]


def test_parquet_data_provider_get_structure():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["1", "2", "3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    with create_temp_parquet_file(data) as parquet_path:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
        )

        # Test get_structure
        structure = provider.get_structure()
        assert "class" in structure
        assert "value" in structure
        assert "sample_id" not in structure  # sample_id should not be included
        assert structure["class"]["category"] == "context"
        assert structure["class"]["type"] == "auto"


def test_parquet_data_provider_get_results_structure():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["S1", "S2", "S3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    results = {
        "m1": pd.DataFrame(
            {
                "sample_id": ["S1", "S2"],
                "predicted_state": ["OK", "KO"],
                "score": [0.9, 0.8],
            }
        ),
        "m2": pd.DataFrame(
            {
                "sample_id": ["S2", "S3"],
                "predicted_state": ["KO", "OK"],
                "score": [0.7, 0.6],
            }
        ),
    }
    with create_temp_parquet_file(data) as parquet_path, create_temp_results_folder(
        results
    ) as results_dir:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            results_parquet_folder_path=results_dir,
        )

        # Test get_results_structure
        results_structure = provider.get_results_structure()
        assert "predicted_state" in results_structure
        assert "score" in results_structure
        assert "sample_id" not in results_structure  # sample_id should not be included
        assert results_structure["predicted_state"]["type"] == "auto"


def test_parquet_data_provider_with_ignored_columns():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["1", "2", "3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
            "extra_column": ["X", "Y", "Z"],
        }
    )
    with create_temp_parquet_file(data) as parquet_path:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            ignored_columns=["extra_column"],
        )

        # Test get_data without the ignored column
        data = provider.get_data(["1", "2"])
        assert data.index.tolist() == ["1", "2"]
        assert "class" in data.columns
        assert "value" in data.columns
        assert "extra_column" not in data.columns


def test_parquet_data_provider_with_ignored_results_columns():
    # Create a temporary parquet file
    data = pd.DataFrame(
        {
            "sample_id": ["S1", "S2", "S3"],
            "class": ["A", "B", "C"],
            "value": [10, 20, 30],
        }
    )
    results = {
        "m1": pd.DataFrame(
            {
                "sample_id": ["S1", "S2"],
                "predicted_state": ["OK", "KO"],
                "score": [0.9, 0.8],
                "extra_result_column": ["X", "Y"],
            }
        ),
        "m2": pd.DataFrame(
            {
                "sample_id": ["S2", "S3"],
                "predicted_state": ["KO", "OK"],
                "score": [0.7, 0.6],
                "extra_result_column": ["Z", "W"],
            }
        ),
    }
    with create_temp_parquet_file(data) as parquet_path, create_temp_results_folder(
        results
    ) as results_dir:
        provider = ParquetDataProvider(
            parquet_path=parquet_path,
            sample_id_column_name="sample_id",
            results_parquet_folder_path=results_dir,
            ignored_results_columns=["extra_result_column"],
        )

        # Test get_model_results without the ignored column
        m1_results = provider.get_model_results("m1", ["S1", "S2"])
        assert m1_results["sample_id"].tolist() == ["S1", "S2"]
        assert "predicted_state" in m1_results.columns
        assert "score" in m1_results.columns
        assert "extra_result_column" not in m1_results.columns
